SystemOrganization addCategory: #'Path-Finder-Callgraph'!
SystemOrganization addCategory: #'Path-Finder-Debugger'!
SystemOrganization addCategory: #'Path-Finder-TestChooser'!
SystemOrganization addCategory: #'Path-Finder-Tracer'!

SCodePaneMorph subclass: #SDebuggerCodePaneMorph
	instanceVariableNames: 'suspiciousColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCodePaneMorph class methodsFor: 'constants' stamp: 'mp 3/25/2011 16:17:19.972'!
codePaneWidth
	^ super codePaneWidth + 15! !

!SDebuggerCodePaneMorph class methodsFor: 'instance creation' stamp: 'mp 11/8/2010 16:19:27.344'!
on: aCallNode color: aColor
	"Label morph will be created later on"
	^ self new
		callNode: aCallNode;
		color: aColor;
		createDefinitionMorph;
		createNodeButtons;
		createObjectExploreButtons;
		yourself! !

!SDebuggerCodePaneMorph class methodsFor: 'instance creation' stamp: 'mp 11/8/2010 16:19:38.859'!
on: aCallNode color: aColor suspiciousColor: anotherColor

	^ (self on: aCallNode color: aColor)
		suspiciousColor: anotherColor;
		createLabelMorph;
		yourself
! !

!SDebuggerCodePaneMorph methodsFor: 'buttons' stamp: 'mp 7/4/2011 17:34:50.627'!
buttonSpec
	| spec |
	spec := super buttonSpec.
	spec add: {#faultLocalizationForMethod. SImageRepository spiderIcon. 'Localize faults at statements'} afterIndex: 2.
	^ spec! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:19:44.05'!
computeOchiaiFor: aDictionary
	| allByteCodes |
	allByteCodes := aDictionary values 
						inject: Set new into: [:set :each | set addAll: each; yourself].
	^ (allByteCodes collect: [:each | self ochiaiForByteCode: each from: aDictionary]) 
		sorted: [:a :b | a value first < b value first]! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:08:41.012'!
displayTest: aTest from: aNumber

	^ 'Processing ', aTest asString, ' out of ', aNumber asString, ' Entry Points.'! !

!SDebuggerCodePaneMorph methodsFor: 'button actions' stamp: 'mp 9/21/2011 12:06:20.914'!
faultLocalizationForMethod
	| byteCodes |
	byteCodes := self getCoveredByteCodes.
	byteCodes := self computeOchiaiFor: byteCodes.
	self highlightAnomaliesAtStatements: byteCodes.! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:47:36.648'!
faultTracer
	self flag: #mpe. "ToDo: Refactor!! Law of Demeter is broken!!"
	^ self callNode callgraphModel pathFinder coverageTracer! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:10:42.082'!
getCoveredByteCodes
	| coveredTests wrapper |
	coveredTests := self faultTracer getCoveringTests: self callNode methodReference.
	[wrapper := SMethodByteCodeCoverageForAllTestsWrapper 
					on: self callNode methodReference.
	wrapper install.
	self runTests: coveredTests forAnalyzing: wrapper]
		ensure: [wrapper uninstall].
	^ wrapper coveredByteCodes! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:05:28.452'!
highlightAnomaliesAtStatements: supicousByteCodes
	| method map stream range |
	method := self callNode compiledMethod.
	map := (DebuggerMethodMap forMethod: method).
	supicousByteCodes do: [:each | 
		stream := (InstructionStream on: method).
		stream pc: each key.
		(stream willReallySend or: [stream willReturn])
			ifTrue: [range := map rangeForPC: each key contextIsActiveContext: true.
					self definition text 
						addAttribute: (TextColor 
										color: (Color 
												h: 120 * (1.0 - each value first) 
												s: each value second
												v: 1.0)) 
						from: range first 
						to: range last;
						addAttribute: TextEmphasis underlined
						from: range first 
						to: range last]].
			self definition selectAll; changed.! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:25:14.785'!
ochiaiForByteCode: aByteCode from: aDictionary
	| coveredTests failuresAtByteCode passedAtByteCode allFailures suspiciousness confidence |
	allFailures := self faultTracer allFailures.
	coveredTests := aDictionary keys 
							select: [:test | (aDictionary at: test) includes: aByteCode].
	failuresAtByteCode := self faultTracer 	failuresFrom: coveredTests.		
	passedAtByteCode := coveredTests size - failuresAtByteCode.
	(allFailures = 0 or: [failuresAtByteCode = 0]) 
		ifTrue: [suspiciousness := 0.0. 
			     confidence := 1.0]
		ifFalse: [suspiciousness := (failuresAtByteCode / 
				((allFailures * (failuresAtByteCode + passedAtByteCode)) sqrt)). 
				confidence := (failuresAtByteCode / allFailures).].
	^ aByteCode -> {suspiciousness. confidence}! !

!SDebuggerCodePaneMorph methodsFor: 'fault localization' stamp: 'mp 9/21/2011 12:10:33.958'!
runTests: aCollectionOfTests forAnalyzing: aWrapper
	aCollectionOfTests 
		do: [:each | aWrapper currentTest: each.
				     each execute] 
		displayingProgress: [:each | self displayTest: each from: aCollectionOfTests size]! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:14:28.07'!
suspiciousColor
	"Answer the value of suspiciousColor"

	^ suspiciousColor! !

!SDebuggerCodePaneMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:14:28.075'!
suspiciousColor: anObject
	"Set the value of suspiciousColor"

	suspiciousColor := anObject! !

SDefinitionModel subclass: #SCallNodeDefinitionModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SCallNodeDefinitionModel class methodsFor: 'as yet unclassified' stamp: 'mp 1/4/2010 16:12'!
on: aCallGraphModel

	^ self new
		callGraphModel: aCallGraphModel! !

!SCallNodeDefinitionModel methodsFor: 'updating' stamp: 'mp 1/31/2010 18:31'!
update: aCallNode

	(aCallNode notNil and: [aCallNode isLiteral]) ifTrue: [^ self].
	aCallNode ifNil:[^ self setClass: nil andSelector: nil]. 
	self setClass: aCallNode clientClass andSelector: aCallNode clientSelector.	
! !

ObjectExplorer subclass: #SObjectExplorer
	instanceVariableNames: 'codePane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SObjectExplorer class methodsFor: 'instance creation' stamp: 'mp 4/27/2010 19:55'!
on: aCodePane

	^ self new
		codePane: aCodePane;
		yourself! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 19:55'!
codePane
	"Answer the value of codePane"

	^ codePane! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 19:55'!
codePane: anObject
	"Set the value of codePane"

	codePane := anObject! !

!SObjectExplorer methodsFor: 'menus' stamp: 'mp 7/7/2010 16:46:23.101'!
genericMenu: aMenu 
	|  menu morph |
	currentSelection
		ifNil: [menu := aMenu.
			menu
				add: '*nothing selected*'
				target: self
				selector: #yourself]
		ifNotNil: [
			morph := SObjectExplorationMorph 
						on: self codePane 
						title: currentSelection asString
						object: currentSelection 
						color: self codePane color.
			self codePane addObjectExplorationMorph: morph].
	^ aMenu! !

!SObjectExplorer methodsFor: 'accessing' stamp: 'mp 4/27/2010 20:15'!
getList
	^ (ObjectExplorerWrapper with: rootObject name: '' model: nil) contents! !

!SObjectExplorer methodsFor: 'as yet unclassified' stamp: 'mp 4/27/2010 18:39'!
noteNewSelection: anObject

	currentSelection := anObject.! !

SActionButtonMorph subclass: #STestResultFeedbackMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-TestChooser'!

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
defaultColor

	^ Color white! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
errorColor

	^ Color r: 230 / 256.0 g: 59 / 256.0 b: 59 / 256.0 ! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:08'!
failureColor

	^ Color r: 245 / 256.0 g: 237 / 256.0 b: 13 / 256.0 ! !

!STestResultFeedbackMorph class methodsFor: 'constants' stamp: 'mp 4/15/2010 16:09'!
passedColor

	^ Color r: 70 / 256.0 g: 189 / 256.0 b: 47 / 256.0 ! !

!STestResultFeedbackMorph methodsFor: 'initialization' stamp: 'mp 4/15/2010 16:15'!
initialize

	super initialize.
	self color: self class defaultColor.! !

!STestResultFeedbackMorph methodsFor: 'updating' stamp: 'mp 5/2/2010 12:52'!
update: aResult

	(aResult isNil or: [aResult isLiteral]) ifTrue: [^ self]. 
	(aResult isKindOf: TestResult) not ifTrue: [^ self].
	
	aResult passed ifNotEmpty: [self color: self class passedColor].
	aResult failures ifNotEmpty: [self color: self class failureColor].
	aResult errors ifNotEmpty: [self color: self class errorColor].! !

BreakpointManager subclass: #SBreakpointManager
	instanceVariableNames: ''
	classVariableNames: 'InstalledNodes'
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:48'!
compilePrototype: aSymbol in: aClass 
	"Compile and return a new method containing a break statement"

	| source node trailer method |
	source := self breakpointMethodSourceFor: aSymbol in: aClass.
	node := aClass compilerClass new
		compile: source
		in: aClass 
		notifying: nil 
		ifFail: [self error: '[breakpoint] unable to install breakpoint'].
	node isNil ifTrue: [^nil].
	trailer := CompiledMethodTrailer new.
	trailer sourceCode: source.
	method := node generate: trailer.
	self installedNodes at: method put: node.
	^ method! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:29'!
installedNodes
	InstalledNodes isNil ifTrue:[InstalledNodes := IdentityDictionary new].
	^InstalledNodes! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:40'!
nukeInstalledNodes

	"self nukeInstalledNodes"
	InstalledNodes := nil.! !

!SBreakpointManager class methodsFor: 'as yet unclassified' stamp: 'mp 3/10/2010 18:41'!
unInstall: breakMethod 

	self installedNodes removeKey: breakMethod.
	super unInstall: breakMethod.! !

Object subclass: #SCallgraphTracer
	instanceVariableNames: 'project selectedCategories callgraphModel profilingMode viewMode active currentId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SCallgraphTracer class methodsFor: 'instance creation' stamp: 'mp 3/27/2012 16:12:29.794'!
on: aSProject

	^ self new
		project: aSProject;
		selectedCategories: aSProject categories! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:33'!
active
	^ active! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:33'!
active: anObject
	active := anObject! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 1/25/2010 15:56'!
addProfileInformation:  aValue node: anId

	(self getNode: anId) profiles at: self profilingMode put: aValue.! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 1/25/2010 15:19'!
addView:  aString to: aSymbol

	self callgraphModel currentNode views at: aSymbol put: aString.! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 3/27/2012 16:36:49.083'!
askForContracts
	self project hasContracts ifFalse: [^ false].
	^ UIManager default 
		chooseFrom: {'Yes'. 'No'}
		values: {true. false}
		lines: #() 
		title: 'Run with Contracts?'! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 11/8/2010 15:17:54.236'!
byteCodeCoverageAt: aCallNode
	| wrapper |
	wrapper := self runTestWithSpecificWrapper: (
					self createByteCodeCoverageWrapperOn: aCallNode).
	^ wrapper coveredByteCodes
! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
callgraphModel
	^ callgraphModel! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
callgraphModel: anObject
	callgraphModel := anObject! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 11/8/2010 15:19:41.521'!
createByteCodeCoverageWrapperOn: aCallNode

	^ SByteCodeCoverageWrapper 
			on: aCallNode clientSelector 
			inClass: aCallNode clientClass 
			waitingCalls: (self callgraphModel numberOfCallsBefore: aCallNode)! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 7/7/2010 17:07:23.694'!
createCodeRootNode: aSourceCodeString

	self callgraphModel rootNode: (SRootCodeCallNode 
								id: self rootId
								code: aSourceCodeString
								fromSender: nil
								model: self callgraphModel).
	self currentId: self rootId.
	self callgraphModel currentNode: self callgraphModel rootNode.! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 7/7/2010 17:21:01.772'!
createDebugWrapperOn: aCallNode

	^ SDebugWrapper 
			on: aCallNode clientSelector 
			inClass: aCallNode clientClass 
			waitingCalls: (self callgraphModel numberOfCallsBefore: aCallNode)! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 7/7/2010 17:21:11.71'!
createExploreWrapper: aSymbol on: aCallNode 
	
	^ SExploreWrapper 
		on: aCallNode clientSelector
		inClass: aCallNode clientClass
		waitingCalls: aCallNode callsBefore
		exploreWhat: aSymbol! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 7/7/2010 17:06:53.646'!
createNewCallNodeOn: aClass with: aSelector receiverType: aTypeClass callsBefore: aNumber

	| node |
	node := SCallNode 
			id: self nextId 
			class: aClass
			methodName: aSelector
			fromSender: self callgraphModel currentNode
			model: self callgraphModel
			receiverType: aTypeClass
			callsBefore: aNumber.
	self callgraphModel currentNode addCall: node.
	self callgraphModel currentNode: node.
	self callgraphModel currentNode 
		selectedView: self viewMode;
		selectedProfile: self profilingMode.
	^ node id! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 1/25/2010 13:37'!
createNewCallgraphOn: aTestCaseClass with: aMethodSelector

	self 
		resetCallgraph;
		createRootNode: aTestCaseClass withTest: aMethodSelector.! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 5/2/2010 12:34'!
createNewCallgraphOnCode: aSourceCodeString
	self 
		resetCallgraph;
		createCodeRootNode: aSourceCodeString.! !

!SCallgraphTracer methodsFor: 'callgraph' stamp: 'mp 7/7/2010 17:07:08.57'!
createRootNode: aTestClass withTest: aSelector

	self callgraphModel rootNode: (SRootTestCallNode 
								id: self rootId
								class: aTestClass
								methodName: aSelector 
								fromSender: nil
								model: self callgraphModel
								receiverType: aTestClass
								callsBefore: 0).
	self currentId: self rootId.
	self callgraphModel currentNode: self callgraphModel rootNode.! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/25/2010 15:22'!
createWrapperOn: aClass and: aSelector addTo: aCollection
	| wrapper |
	wrapper := self getCallWrapperClass on: aSelector inClass: aClass withTracer: self.
	wrapper ifNotNil: [
		wrapper selectedView: self viewMode.
		aCollection add: wrapper.
		wrapper install].! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:01'!
createWrappers
	| wrappers |
	wrappers := OrderedCollection new.
	self selectedClasses do: [:klass | 
			klass selectors do: [:selector | 
				self createWrapperOn: klass and: selector addTo: wrappers].		
			klass class selectors do: [:selector | 
				self createWrapperOn: klass class and: selector addTo: wrappers]].
	^ wrappers! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:59'!
currentId
	^ currentId! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:59'!
currentId: anObject
	currentId := anObject! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 7/7/2010 16:59:59.875'!
debugAt:	aCallNode
	
	| wrapper |
	(self callgraphModel numberOfCallsBefore: aCallNode) = 0
		ifTrue: [
			[SBreakpointManager installInClass: aCallNode clientClass selector: aCallNode clientSelector.
			self runRootNode] ensure: [
				SBreakpointManager unInstall:  aCallNode clientClass >> aCallNode clientSelector]]
		ifFalse: [
			[wrapper := self createDebugWrapperOn: aCallNode.
			wrapper install.
			self runRootNode] ensure: [
				SBreakpointManager unInstall:  aCallNode clientClass >> aCallNode clientSelector.
				wrapper uninstall]]! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'bs 7/17/2010 14:25:54.337'!
explore: aSymbol on: aCallNode

	| instVars |
	instVars := self runTestWithSpecificWrapper: (self createExploreWrapper: aSymbol on: aCallNode).
	^ instVars objects! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 7/7/2010 17:21:52.072'!
getCallWrapperClass

	(#(% ms) includes: self profilingMode)
		ifTrue: [^ SProfilingWrapper].
	(#(identities objects) includes: self viewMode)
		ifTrue: [^ SViewWrapper].
	^ SCallWrapper! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 2/6/2010 16:55'!
getNode: aNumber

	self callgraphModel currentNode id = aNumber
		ifTrue: [^ self callgraphModel currentNode]
		ifFalse: [^ self callgraphModel findNodeInSubGraph: aNumber startNode: self callgraphModel rootNode].! !

!SCallgraphTracer methodsFor: '-- all --' stamp: 'mp 3/5/2010 17:53'!
initialize

	self 
		stopTracing;
		viewMode: #classes;
		profilingMode: #normal;
		currentId: -1.! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 18:08'!
isActive
	
	^ self active! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:26'!
nextId
	
	self currentId: self currentId + 1.
	^ self currentId! !

!SCallgraphTracer methodsFor: 'profiling' stamp: 'mp 1/25/2010 16:08'!
postProfilingCounts

	| bag |
	bag := Bag new.
	self callgraphModel 
		traverseCallGraph: self callgraphModel rootNode 
		with: [:node | bag add: node clientClass -> node clientSelector.
			node profiles 
				at: #counts 
				put: '#', (bag occurrencesOf: node clientClass -> node clientSelector) asString].! !

!SCallgraphTracer methodsFor: 'profiling' stamp: 'mp 1/25/2010 16:07'!
postProfilingMilliSeconds

	| duration |
	self callgraphModel 
		traverseCallGraph: self callgraphModel rootNode 
		with: [:node | 
			duration := node profiles at: #ms.
			node profiles at: #ms put: duration asMilliSeconds asString, 'ms'].! !

!SCallgraphTracer methodsFor: 'profiling' stamp: 'mp 2/6/2010 22:13'!
postProfilingPercentage

	| fullDuration duration |
	fullDuration := self callgraphModel startingCallNodes 
					inject: 0 into: [:sum :each | sum := sum + (each profiles at: #%) asNanoSeconds].
					
	self callgraphModel 
		traverseCallGraph: self callgraphModel rootNode 
		with: [:node | 
			(node profiles includesKey: #%) 
				ifTrue: [duration := (node profiles at: #%) asNanoSeconds.
					node profiles at: #% put: ((duration / fullDuration) * 100.0) rounded asString, '%']].! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:11'!
profilingMode
	^ profilingMode ifNil: [profilingMode := #normal]! !

!SCallgraphTracer methodsFor: 'public API' stamp: 'mp 1/18/2010 16:31'!
profilingMode: anObject
	profilingMode := anObject! !

!SCallgraphTracer methodsFor: 'profiling' stamp: 'mp 1/25/2010 15:47'!
profilingPostProcessing

	self profilingMode == #ms
		ifTrue: [self postProfilingMilliSeconds].
	self profilingMode == #%
		ifTrue: [self postProfilingPercentage].
	self profilingMode == #counts
		ifTrue: [self postProfilingCounts].! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:11:51.655'!
project
	"Answer the value of project"

	^ project! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:11:51.668'!
project: anObject
	"Set the value of project"

	project := anObject! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:17'!
removeWrappers: aWrapperCollection
	
	aWrapperCollection do: [:each | each uninstall].! !

!SCallgraphTracer methodsFor: 'public API' stamp: 'mp 4/15/2010 14:10'!
resetCallgraph

	self currentId: -1.
	self callgraphModel 
		reset; 
		refresh.! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 2/6/2010 22:24'!
returnFromNode: id

	| node |
	self callgraphModel currentNode id = id
		ifTrue: [self callgraphModel currentNode: self callgraphModel currentNode fromSender]
		ifFalse: [node := self callgraphModel findNodeInSubGraph: id startNode: self callgraphModel rootNode.
					self callgraphModel currentNode: node fromSender].! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:46'!
rootId

	^ 0! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 5/2/2010 12:07'!
runRootNode
	^ self callgraphModel rootNode run! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 3/27/2012 16:56:32.626'!
runRootNodeWithContracts
	^ [[self runRootNode] runWithContracts: self project projectName]
		on: DCContractViolation
		do: [:exception | 
			self callgraphModel currentNode violations add: exception.
			exception resume]! !

!SCallgraphTracer methodsFor: 'specific runs' stamp: 'mp 7/12/2010 11:37:26.051'!
runTestWithSpecificWrapper: aWrapper

	aWrapper install.
	[self runRootNode]
		ensure: [aWrapper uninstall.
			      ^ aWrapper].! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
selectedCategories
	^ selectedCategories! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:29'!
selectedCategories: anObject
	selectedCategories := anObject! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 16:30'!
selectedClasses

	^ Smalltalk allClasses select: [:aClass | self selectedCategories includes: aClass category]! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:32'!
startTracing

	self active: true.! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 1/18/2010 17:32'!
stopTracing

	self active: false.! !

!SCallgraphTracer methodsFor: 'tracing' stamp: 'mp 3/27/2012 16:42:24.788'!
trace
	| result wrappers |
	wrappers := self createWrappers.
	self startTracing.
	result := self askForContracts
		ifTrue: [self runRootNodeWithContracts]
		ifFalse: [self runRootNode].
	self 
		stopTracing;
		removeWrappers: wrappers;
		profilingPostProcessing.
	self callgraphModel refresh.
	^ result! !

!SCallgraphTracer methodsFor: 'public API' stamp: 'mp 3/27/2012 16:21:01.35'!
traceSourceCode: aSourceCodeString

	self createNewCallgraphOnCode: aSourceCodeString.
	^ self trace! !

!SCallgraphTracer methodsFor: 'public API' stamp: 'mp 3/27/2012 16:21:05.735'!
traceTest: aTestCaseClass method: aMethodSelector

	self createNewCallgraphOn: aTestCaseClass with: aMethodSelector.
	^ self trace! !

!SCallgraphTracer methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:18'!
viewMode
	^ viewMode ifNil: [viewMode := #classes]! !

!SCallgraphTracer methodsFor: 'public API' stamp: 'mp 1/18/2010 16:31'!
viewMode: anObject
	viewMode := anObject! !

Object subclass: #SNode
	instanceVariableNames: 'callgraphModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

SNode subclass: #SMethodNode
	instanceVariableNames: 'clientClass clientSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

SMethodNode subclass: #SCallNode
	instanceVariableNames: 'id fromSender nextCalls selectedView selectedProfile views profiles objects callsBefore violations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SCallNode class methodsFor: 'instance creation' stamp: 'mp 6/15/2010 16:03'!
id: aNumber class: aClass methodName: aSelector fromSender: aCallNode model: aCallgraphModel receiverType: aTypeClass callsBefore: aCallsNumber

	^ (self forClass: aClass withSelector: aSelector model: aCallgraphModel)
		id: aNumber;
		fromSender: aCallNode;
		callsBefore: aCallsNumber;
		setReceiverType: aTypeClass;
		initClassView;
		yourself! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
addCall: aCallNode

	self nextCalls add: aCallNode.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 6/15/2010 15:59'!
callsBefore
	"Answer the value of callsBefore"

	^ callsBefore! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 6/15/2010 15:59'!
callsBefore: anObject
	"Set the value of callsBefore"

	callsBefore := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 4/24/2010 18:43'!
clearObjectCache

	self objects: nil.! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 4/24/2010 18:51'!
explore: aSymbol
	^ self objects 
		at: aSymbol 
		ifAbsentPut: [self tracer explore: aSymbol on: self]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
fromSender
	^ fromSender! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
fromSender: anObject
	fromSender := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 1/25/2010 13:59'!
getCallStack

	| callStack currentNode |
	callStack := OrderedCollection new.
	currentNode := self.
	callStack addFirst: currentNode.
	[currentNode fromSender notNil] whileTrue: [
		currentNode := currentNode fromSender.
		callStack addFirst: currentNode].
	^ callStack! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
id
	^ id! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
id: anObject
	id := anObject! !

!SCallNode methodsFor: 'initialize-release' stamp: 'mp 5/5/2010 13:41'!
initClassView

	self views at: #classes put: self clientClass.! !

!SCallNode methodsFor: 'initialize-release' stamp: 'mp 5/5/2010 13:37'!
initialize

	self id: -1.
	self nextCalls: OrderedCollection new.
	self views: Dictionary new.
	self profiles: Dictionary new.
	self selectedView: #classes.
	self selectedProfile: #normal.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
nextCalls
	^ nextCalls! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:41'!
nextCalls: anObject
	nextCalls := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 4/19/2010 20:38'!
objects
	"Answer the value of objects"

	^ objects ifNil: [objects := Dictionary new]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 4/19/2010 20:37'!
objects: anObject
	"Set the value of objects"

	objects := anObject! !

!SCallNode methodsFor: 'printing' stamp: 'mp 4/24/2010 14:04'!
printNodeId

	^ '(', self id asString, ')'	! !

!SCallNode methodsFor: 'printing' stamp: 'mp 1/25/2010 13:57'!
printProfile

	self selectedProfile == #normal ifTrue: [^''].
	^ '[',  (self profiles at: self selectedProfile), ']'! !

!SCallNode methodsFor: 'printing' stamp: 'mp 4/24/2010 14:04'!
printSelector

	^ '>>', self clientSelector! !

!SCallNode methodsFor: 'printing' stamp: 'mp 10/31/2010 12:11:29.179'!
printString

	 ^ (self callgraphModel printInfosForNode: self), 
		self printProfile, 
		self printView, 
		self printSelector

	! !

!SCallNode methodsFor: 'printing' stamp: 'mp 2/1/2012 11:21:55.809'!
printView

	| result |
	(self selectedView == #classes 
		and: [((self views at: #classes) asString = (self views at: #receiver) asString) not]) 
			ifTrue: [^ (self views at: #receiver) asString, 
						'(', (self views at: #classes) asString, ')'].
	self selectedView == #identities
		ifTrue: [result := (self views at: #receiver) asString, 
					'(', (self views at: #identities) asString, ')'.
				((self views at: #classes) asString = 
					(self views at: #receiver) asString) not
						ifTrue: [result := result, '(', (self views at: #classes) asString, ')'].
				^ result	].
	^ (self views at: self selectedView) asString.

	! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
profiles
	"Answer the value of profiles"

	^ profiles! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
profiles: anObject
	"Set the value of profiles"

	profiles := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedProfile
	"Answer the value of selectedProfile"

	^ selectedProfile! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedProfile: anObject
	"Set the value of selectedProfile"

	selectedProfile := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedView
	"Answer the value of selectedView"

	^ selectedView! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
selectedView: anObject
	"Set the value of selectedView"

	selectedView := anObject! !

!SCallNode methodsFor: 'callgraph' stamp: 'mp 1/25/2010 13:41'!
setAsCurrentSelection

	self callgraphModel 
		newSelectedItem: self;
		changed: #getSelectedPath.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 5/5/2010 13:25'!
setReceiverType: aClass

		self views at: #receiver put: aClass.! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 4/24/2010 18:51'!
tracer

	^ self callgraphModel tracer! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
views
	"Answer the value of views"

	^ views! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 1/25/2010 13:43'!
views: anObject
	"Set the value of views"

	views := anObject! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:56:00.054'!
violations
	"Answer the value of violations"

	^ violations ifNil: [violations := OrderedCollection new]! !

!SCallNode methodsFor: 'accessing' stamp: 'mp 3/27/2012 16:55:17.287'!
violations: anObject
	"Set the value of violations"

	violations := anObject! !

SCallNode subclass: #SRootCallNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SRootCallNode methodsFor: 'printing' stamp: 'mp 4/27/2010 20:28'!
printString

	 ^ 'Entry Point'! !

!SRootCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:28'!
run

	self subclassResponsibility.! !

SRootCallNode subclass: #SRootCodeCallNode
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SRootCodeCallNode class methodsFor: 'instance creation' stamp: 'mp 5/2/2010 12:31'!
id: aNumber code: aSourceCodeString fromSender: aCallNode model: aCallgraphModel

	^ (self forClass: nil withSelector: nil model: aCallgraphModel)
		id: aNumber;
		fromSender: aCallNode;
		code: aSourceCodeString;
		initClassView;
		yourself! !

!SRootCodeCallNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:30'!
code
	"Answer the value of code"

	^ code! !

!SRootCodeCallNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:30'!
code: anObject
	"Set the value of code"

	code := anObject! !

!SRootCodeCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:55'!
numArgsOfMethod
	"Custom code has no method and no arguments"
	^ 0! !

!SRootCodeCallNode methodsFor: 'running' stamp: 'mp 5/2/2010 12:51'!
run
	^ Compiler evaluate: self code! !

SRootCallNode subclass: #SRootTestCallNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SRootTestCallNode methodsFor: 'running' stamp: 'mp 9/22/2011 16:25:26.442'!
run
	| result |
	 self clientClass resources do: [:each |
		each isAvailable ifFalse: [ each signalInitializationError ]].
	[result := (self clientClass selector: self clientSelector) runWithoutTimeout] 
		ensure: [self clientClass resources do: [:each | each reset ]].
	^ result! !

!SMethodNode class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 13:50'!
forClass: aClass withSelector: aSelector model: aCallgraphModel

	^ (self for: aCallgraphModel)
			clientClass: aClass;
			clientSelector: aSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientClass
	^ clientClass! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientClass: anObject
	clientClass := anObject! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientSelector
	^ clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 1/3/2010 14:04'!
clientSelector: anObject
	clientSelector := anObject! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 4/24/2010 15:04'!
compiledMethod

	^ self clientClass >> self clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 4/24/2010 15:03'!
methodReference

	^ MethodReference class: self clientClass selector: self clientSelector! !

!SMethodNode methodsFor: 'accessing' stamp: 'mp 5/2/2010 12:54'!
numArgsOfMethod

	^ self compiledMethod numArgs! !

!SNode class methodsFor: 'instance creation' stamp: 'mp 1/18/2010 16:43'!
for: aCallgraphModel

	^ self new
		callgraphModel: aCallgraphModel! !

!SNode methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:43'!
callgraphModel
	^ callgraphModel! !

!SNode methodsFor: 'accessing' stamp: 'mp 1/18/2010 16:43'!
callgraphModel: anObject
	callgraphModel := anObject! !

Object subclass: #STestChooserModel
	instanceVariableNames: 'tracer test result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-TestChooser'!

!STestChooserModel class methodsFor: 'instance creation' stamp: 'mp 4/15/2010 15:32'!
on: aTracer

	^ self new
		tracer: aTracer;
		yourself! !

!STestChooserModel methodsFor: 'private' stamp: 'mp 5/2/2010 12:25'!
analyzeTest

	| result |
	result := self test isString
		ifTrue: [self tracer traceSourceCode: self test]
		ifFalse: [self tracer traceTest: self test actualClass method: self test methodSymbol].
	self result: result.! !

!STestChooserModel methodsFor: 'spec commands' stamp: 'mp 4/15/2010 15:39'!
isTestAvailable

	^ self test notNil! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
normalRun

	self setProfilingStateTo: #normal.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
normalRunWithCounts	
	
	self setProfilingStateTo: #counts.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
profileRunWithMs

	self setProfilingStateTo: #ms.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
profileRunWithPercentage

	self setProfilingStateTo: #%.! !

!STestChooserModel methodsFor: 'private' stamp: 'mp 1/25/2010 13:23'!
resetCallgraph

	self tracer ifNil: [^ self].
	self tracer resetCallgraph.! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 12:39'!
result
	"Answer the value of result"

	^ result! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 16:21'!
result: anObject
	"Set the value of result"

	result := anObject.
	self changed: result.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectClassesView

	self setViewStateTo: #classes.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectIdentityView

	self setViewStateTo: #identities.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:58'!
selectObjectView

	self setViewStateTo: #objects.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 5/2/2010 12:21'!
setCurrentCode: aSourceCodeString

	self test: aSourceCodeString.
	self changed: #isTestAvailable.
	aSourceCodeString 
		ifNil: [self resetCallgraph.
	 		   ^ self].
	self analyzeTest.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 4/15/2010 15:41'!
setCurrentTest: aMethodReference

	self test: aMethodReference.
	self changed: #isTestAvailable.
	aMethodReference 
		ifNil: [self resetCallgraph.
	 		   ^ self].
	self analyzeTest.	! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:56'!
setProfilingStateTo: aSymbol 
	
	self tracer profilingMode: aSymbol.! !

!STestChooserModel methodsFor: 'actions' stamp: 'mp 1/21/2010 12:57'!
setViewStateTo: aSymbol 
	
	self tracer viewMode: aSymbol.! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 15:38'!
test
	"Answer the value of test"

	^ test! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 4/15/2010 15:38'!
test: anObject
	"Set the value of test"

	test := anObject! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 09:59'!
tracer
	"Answer the value of tracer"

	^ tracer! !

!STestChooserModel methodsFor: 'accessing' stamp: 'mp 1/21/2010 09:59'!
tracer: anObject
	"Set the value of tracer"

	tracer := anObject! !

STreeModel subclass: #SCallgraphModel
	instanceVariableNames: 'rootNode currentNode history currentHistoryNode tracer searchQuery pathFinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Callgraph'!

!SCallgraphModel class methodsFor: 'instance creation' stamp: 'mp 3/25/2011 12:46:16.783'!
on: aTracer

	^ self on: aTracer pathFinder: nil! !

!SCallgraphModel class methodsFor: 'instance creation' stamp: 'mp 3/25/2011 12:45:54.853'!
on: aTracer pathFinder: aPathFinder

	^ self new
		tracer: aTracer;
		pathFinder: aPathFinder.! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/25/2010 16:19'!
addToHistory: aNode

	aNode ifNotNil: [
		(self history notEmpty and: [((self history at: self currentHistoryNode) id = aNode id)])
			ifTrue: [^ self].			
		(self history size = self currentHistoryNode) not
			ifTrue: [self history: (self history copyUpTo: (self history at: self currentHistoryNode + 1))].
		self history add: aNode.
		self currentHistoryNode: self history size].
	self refreshHistoryButtons.! !

!SCallgraphModel methodsFor: 'menu' stamp: 'mp 4/15/2010 13:03'!
browseMethod

	self hasSelectedItem 
		ifTrue:[^ Browser fullOnClass: self selectedItem clientClass 
					       selector: self selectedItem clientSelector]! !

!SCallgraphModel methodsFor: 'menu' stamp: 'mp 3/4/2010 14:50'!
callGraphMenu: aMenu

	^ aMenu
		title: 'Node and Tree Actions';
		add: 'Browse' action: #browseMethod;
		add: 'Debug' action: #debugMethod;
		addLine;
		add: 'Expand Tree' action: #expandSelectedNode;
		yourself.! !

!SCallgraphModel methodsFor: 'searching' stamp: 'mp 1/16/2010 18:42'!
clearHighlighting

	self highlightedItems: nil! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 14:52'!
currentHistoryNode
	^ currentHistoryNode! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 14:52'!
currentHistoryNode: anObject
	currentHistoryNode := anObject! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
currentNode
	^ currentNode! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
currentNode: anObject
	currentNode := anObject! !

!SCallgraphModel methodsFor: 'menu' stamp: 'mp 1/25/2010 16:44'!
debugMethod

	self hasSelectedItem ifTrue: [self tracer debugAt: self selectedItem].! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/29/2010 10:32'!
expandNode: aNode

	aNode setAsCurrentSelection.
	self changed: #expandAllOfSelectedNode.! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/29/2010 10:15'!
expandRootNode

	self expandNode: self rootNode.! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 3/4/2010 14:46'!
expandSelectedNode

	self selectedItem
		ifNil: [self expandNode: self rootNode]
		ifNotNil: [self expandNode: self selectedItem]
	! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/3/2010 14:59'!
findNodeInSubGraph: aNumber startNode: aNode

	| lastNode |
	aNode nextCalls do: [:each | 
		each id = aNumber ifTrue: [^ each].
		each id < aNumber ifTrue: [lastNode := each].
		each id > aNumber ifTrue: [
			lastNode	ifNotNil: [^ self findNodeInSubGraph: aNumber startNode: lastNode]]].
	lastNode ifNotNil: [^ self findNodeInSubGraph: aNumber startNode: lastNode].
	^ nil! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/25/2011 16:07:06.532'!
getAdditionalInformationFromMap: aCallNode
	^ self pathFinder coverageTracer 
			ifNotNil: [:tracer | 
				tracer getAdditionalInfoFromMapFor: (MethodReference 
												class: aCallNode clientClass 
												selector: aCallNode clientSelector)].! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 1/3/2010 15:09'!
getChildren: aNode

	^ aNode nextCalls! !

!SCallgraphModel methodsFor: 'path view' stamp: 'mp 3/7/2012 17:22:30.127'!
getClassesFromTrace
	| classes |
	classes := Set new. 
	self traverseCallGraph: self rootNode with: [:each | 
		each clientClass isMeta
			ifTrue: [classes add: each clientClass theNonMetaClass]
			ifFalse: [classes add: each clientClass]].
	^ classes! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/25/2011 15:05:00.123'!
getColorFromMap: aCallNode
	^ self pathFinder coverageTracer 
			ifNotNil: [:tracer | 
				tracer getColorFromMapFor: (MethodReference 
												class: aCallNode clientClass 
												selector: aCallNode clientSelector)].! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/25/2011 15:03:08.867'!
getIcon: aCallNode
	| icon color |	
	color := self getColorFromMap: aCallNode.
	(color isNil or: [color = Color white]) ifTrue: [^ nil].
	icon := Form extent: 15@15 depth: 32.
	icon fillColor: color.
	^ icon! !

!SCallgraphModel methodsFor: 'path view' stamp: 'mp 3/7/2012 18:25:29.524'!
getMethodsFromTrace
	| methods |
	methods := Set new.
	self traverseCallGraph: self rootNode 
		 with: [:each | methods add: each methodReference].
	^ methods! !

!SCallgraphModel methodsFor: 'searching' stamp: 'mp 3/4/2010 13:57'!
getSearchQuery

	self searchQuery ifNil: [^ ''].
	^ self searchQuery! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/1/2010 17:39'!
getSelectedPath

	^ self selectedItem getCallStack! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 1/3/2010 15:10'!
hasChildren: aNode

	aNode ifNil: [^ false].
	^ aNode nextCalls notEmpty! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 13:25'!
history
	^ history! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/15/2010 13:25'!
history: anObject
	history := anObject! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:01'!
historyBack

	self currentHistoryNode: self currentHistoryNode - 1.
	(self history at: self currentHistoryNode) setAsCurrentSelection.
	self refreshHistoryButtons.! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/15/2010 15:12'!
historyBackButtonEnabled

	^ (self currentHistoryNode <= 1) not! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:01'!
historyPrevious

	self currentHistoryNode: self currentHistoryNode + 1.
	(self history at: self currentHistoryNode) setAsCurrentSelection.
	self refreshHistoryButtons.! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/15/2010 15:10'!
historyPreviousButtonEnabled

	^ self history notEmpty and: [(self currentHistoryNode = self history size) not]! !

!SCallgraphModel methodsFor: 'initialize-release' stamp: 'mp 1/25/2010 13:29'!
initialize
	
	self reset.! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/7/2012 17:11:28.962'!
isGraphAvailable

	^ self rootNode notNil! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 1/15/2010 13:28'!
newSelectedItem: aNode

	self
		selectedItem: aNode;
		addToHistory: aNode;
		changed: aNode.! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/8/2010 16:09'!
numberOfCallsBefore: aCallNode

	| numberOfCallsBefore |
	numberOfCallsBefore := 0.
	self traverseCallGraph: self rootNode 
		with: [:aNode | 
			((aNode clientClass == aCallNode clientClass) and: 
			[aNode clientSelector == aCallNode clientSelector])
				ifTrue: [
					aNode id < aCallNode id 
						ifTrue: [numberOfCallsBefore := numberOfCallsBefore + 1]]]
		abort: [:aNode | aNode id > aCallNode id].
	^ numberOfCallsBefore! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/7/2012 17:23:05.619'!
openPathViewOnTrace
	self openPathViewWith: self getClassesFromTrace.! !

!SCallgraphModel methodsFor: 'path view' stamp: 'ag 3/14/2012 11:49:45.554'!
openPathViewWith: aSetOfClasses
	| morph |
	morph := SPathView open classDiagramMorph.
	morph addClasses: aSetOfClasses.
	self getMethodsFromTrace 
		collect: [:each | each actualClass -> each methodSymbol] 
		thenDo: [:each | morph pinMethod: each].
	morph expandAndToggleLabels.
	
	"Perform the layout in a separate thread, so that the morphs already have their positions and extents applied."
	"TODO find a clean way in Morph to defer the layout until the PasteUpMorph with the graph is fully initialized."
	[ (Delay forMilliseconds: 300) wait.
	morph layoutGraph ]
		fork! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:45:31.191'!
pathFinder
	"Answer the value of pathFinder"

	^ pathFinder! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:45:31.196'!
pathFinder: anObject
	"Set the value of pathFinder"

	pathFinder := anObject! !

!SCallgraphModel methodsFor: 'printing' stamp: 'mp 10/31/2010 12:19:32.481'!
printInfosForNode: aCallNode

	"template method for printing additional information from the entire graph"
	^ ''! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/3/2010 14:21'!
refresh

	self changed: #rootCallNodes.! !

!SCallgraphModel methodsFor: 'history' stamp: 'mp 1/15/2010 16:00'!
refreshHistoryButtons

	self
		changed: #historyBackButtonEnabled;
		changed: #historyPreviousButtonEnabled.! !

!SCallgraphModel methodsFor: 'initialize-release' stamp: 'mp 3/4/2010 13:10'!
reset
	
	self 
		rootNode: nil;
		currentNode: nil;
		history: OrderedCollection new;
		currentHistoryNode: 0;
		newSelectedItem: nil;
		highlightedItems: nil.! !

!SCallgraphModel methodsFor: 'spec commands' stamp: 'mp 3/1/2010 17:41'!
rootCallNodes

	self rootNode ifNil: [^ ''].
	^ {self rootNode}! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
rootNode
	^ rootNode! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/3/2010 13:45'!
rootNode: anObject
	rootNode := anObject! !

!SCallgraphModel methodsFor: 'searching' stamp: 'mp 3/9/2010 14:29'!
searchNode: aText

	| foundedNodes |
	self clearHighlighting.
	self rootNode ifNotNil: [
		self searchQuery: aText asString.
		foundedNodes := OrderedCollection new.
		self traverseCallGraph: self rootNode 
			with: [:aNode | (aNode asString includesSubString: self searchQuery) 
							ifTrue: [foundedNodes add: aNode]].
		self 
			highlightedItems: foundedNodes;
			changed: #highlightItem:;
			selectNextSearchResult: foundedNodes.
		^ foundedNodes]
	! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/4/2010 13:56'!
searchQuery
	"Answer the value of searchQuery"

	^ searchQuery! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/4/2010 13:56'!
searchQuery: anObject
	"Set the value of searchQuery"

	searchQuery := anObject! !

!SCallgraphModel methodsFor: 'searching' stamp: 'mp 3/4/2010 13:27'!
selectNextSearchResult: aCollection
	| nextSearchNode |
	aCollection ifEmpty: [^ self].
	self selectedItem 
		ifNil: [nextSearchNode := aCollection first]
		ifNotNil: [nextSearchNode := aCollection 
									detect: [:each | each id > self selectedItem id] 
									ifNone: [aCollection first]].
	nextSearchNode setAsCurrentSelection.! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/4/2010 15:51'!
startingCallNodes

	^ self rootNode nextCalls! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:23'!
tracer
	^ tracer! !

!SCallgraphModel methodsFor: 'accessing' stamp: 'mp 1/18/2010 17:23'!
tracer: anObject
	tracer := anObject! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/4/2010 15:44'!
traverseCallGraph: aNode with: aBlock
	"Prefix Order"
	aNode nextCalls do: [:each | 
		aBlock value: each.
		self traverseCallGraph: each with: aBlock].! !

!SCallgraphModel methodsFor: 'graph functions' stamp: 'mp 1/8/2010 16:00'!
traverseCallGraph: aNode with: aBlock abort: anotherBlock
	"Prefix Order"
	aNode nextCalls do: [:each | 
		aBlock value: each.
		(anotherBlock value: each) 
			ifFalse: [self traverseCallGraph: each with: aBlock abort: anotherBlock]].! !

SCallgraphModel subclass: #SDebuggerCallgraphModel
	instanceVariableNames: 'suspiciousMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCallgraphModel methodsFor: 'icons' stamp: 'mp 3/25/2011 12:18:12.702'!
getIcon: aCallNode
	| icon suspiciousMethod |
	suspiciousMethod := self getSuspiciousnessAndConfidenceFor: aCallNode.
	suspiciousMethod ifNil: [^ nil].
	icon := Form extent: 15@15 depth: 32.
	icon fillColor: (Color 
					h: (1.0 - suspiciousMethod value first) * 120 
					s: suspiciousMethod value second 
					v: 1.0).
	^ icon! !

!SDebuggerCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:25:43.124'!
getSuspiciousnessAndConfidenceFor: aCallNode
	| methodRef |
	methodRef := MethodReference 
					class: aCallNode clientClass selector: aCallNode clientSelector.
	^ self suspiciousMethods 
		detect: [:each | each key = methodRef] 
		ifNone: [nil].! !

!SDebuggerCallgraphModel methodsFor: 'printing' stamp: 'mp 10/31/2010 13:28:51.743'!
printInfosForNode: aCallNode
	| suspiciousMethod |
	suspiciousMethod := self getSuspiciousnessAndConfidenceFor: aCallNode.
	suspiciousMethod ifNil: [^ ''].
	^  '{', (suspiciousMethod value first * 100) asInteger asString, '%} '! !

!SDebuggerCallgraphModel methodsFor: 'searching' stamp: 'mp 10/31/2010 12:56:29.143'!
searchNode: aText
	| query |
	"Allows search queries such as {> 90 meaning all suspicious methods 
	with a value larger than 90 %"
	query := aText asString splitInteger.
	query first withBlanksTrimmed caseOf: {
	['{>'] -> [^ self searchSuspiciousNodes: query second operator: #>].
	['{<'] -> [^ self searchSuspiciousNodes: query second operator: #<].
	['{='] -> [^ self searchSuspiciousNodes: query second operator: #=].
	}
	otherwise: [^ super searchNode: aText].
	! !

!SDebuggerCallgraphModel methodsFor: 'searching' stamp: 'mp 10/31/2010 13:01:27.976'!
searchSuspiciousNodes: aNumber operator: aSymbol
	| foundedNodes nodeString |
	self clearHighlighting.
	self rootNode ifNotNil: [
		foundedNodes := OrderedCollection new.
		self traverseCallGraph: self rootNode 
			with: [:aNode | 
				nodeString := aNode asString.
				((nodeString first = ${) and: 
					[nodeString splitInteger second perform: aSymbol with: aNumber])
						ifTrue: [foundedNodes add: aNode]].
		self 
			highlightedItems: foundedNodes;
			changed: #highlightItem:;
			selectNextSearchResult: foundedNodes.
		^ foundedNodes].! !

!SDebuggerCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:47:47.326'!
suspiciousMethods
	"Answer the value of suspiciousMethods"

	^ suspiciousMethods ifNil: [suspiciousMethods := 
								self pathFinder coverageTracer mostSuspiciousMethods]! !

!SDebuggerCallgraphModel methodsFor: 'accessing' stamp: 'mp 3/25/2011 12:25:31.106'!
suspiciousMethods: anObject
	"Set the value of suspiciousMethods"

	suspiciousMethods := anObject! !

MwMethodWrapper subclass: #SCallWrapper
	instanceVariableNames: 'ids tracer callsBefore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SCallWrapper class methodsFor: 'testing' stamp: 'mp 4/1/2011 15:51:33.932'!
canWrap: aSymbol inClass: aClass
	(aClass inheritsFrom: ACUserStory) ifTrue: [^ false].
	^ super canWrap: aSymbol inClass: aClass! !

!SCallWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 14:25'!
on: aSymbol inClass: aClass withTracer: aTracer

	| wrapper |
	wrapper := self on: aSymbol inClass: aClass.
	wrapper ifNotNil: [wrapper tracer: aTracer].
	^ wrapper! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 1/25/2010 18:10'!
afterMethod

	self tracer returnFromNode: self ids removeLast.! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 6/15/2010 16:09'!
beforeMethod: anObject
	self ids add: (self tracer 
				createNewCallNodeOn: self wrappedClass 
				with: self clientSelector
				receiverType: anObject class
				callsBefore: self callsBefore).
	self callsBefore: self callsBefore + 1.! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 6/15/2010 16:08'!
callsBefore
	"Answer the value of callsBefore"

	^ callsBefore! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 6/15/2010 16:08'!
callsBefore: anObject
	"Set the value of callsBefore"

	callsBefore := anObject! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:09'!
ids
	"Answer the value of ids"

	^ ids! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:09'!
ids: anObject
	"Set the value of ids"

	ids := anObject! !

!SCallWrapper methodsFor: '-- all --' stamp: 'mp 6/15/2010 16:09'!
initialize

	self ids: OrderedCollection new.
	self callsBefore: 0.! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 15:22'!
selectedView: aSymbol! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:24'!
tracer
	"Answer the value of tracer"

	^ tracer! !

!SCallWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:24'!
tracer: anObject
	"Set the value of tracer"

	tracer := anObject! !

!SCallWrapper methodsFor: 'evaluating' stamp: 'mp 5/5/2010 12:18'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	self 
		beforeMethod;
		beforeMethod: anObject.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] ensure: [self afterMethod]! !

SCallWrapper subclass: #SViewWrapper
	instanceVariableNames: 'selectedView'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

SViewWrapper subclass: #SProfilingWrapper
	instanceVariableNames: 'stopwatchs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SProfilingWrapper methodsFor: 'evaluating' stamp: 'mp 1/26/2010 09:29'!
afterMethod

	| stopwatch |
	stopwatch := self stopwatchs removeLast.
	stopwatch suspend.
	self tracer addProfileInformation:  stopwatch duration node: self ids last.
	super afterMethod.! !

!SProfilingWrapper methodsFor: 'evaluating' stamp: 'mp 1/26/2010 09:28'!
beforeMethod

	super beforeMethod.
	self stopwatchs add: Stopwatch new.
	self stopwatchs last activate.! !

!SProfilingWrapper methodsFor: '-- all --' stamp: 'mp 1/26/2010 09:28'!
initialize

	super initialize.
	self stopwatchs: OrderedCollection new.! !

!SProfilingWrapper methodsFor: 'accessing' stamp: 'mp 1/26/2010 09:28'!
stopwatchs
	"Answer the value of stopwatchs"

	^ stopwatchs! !

!SProfilingWrapper methodsFor: 'accessing' stamp: 'mp 1/26/2010 09:28'!
stopwatchs: anObject
	"Set the value of stopwatchs"

	stopwatchs := anObject! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 6/28/2010 22:28'!
getViewString: anObject

	self selectedView == #identities
		ifTrue: ["^ self wrappedClass asString, '(', anObject identityHash, ')'"
			^ anObject identityHash].		
	self selectedView == #objects
		ifTrue: [^ [anObject asString] ifError: ['Error cannot print object']].
	self selectedView == #classes
		ifTrue: [^ self wrappedClass asString].
! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:38'!
selectedView
	"Answer the value of selectedView"

	^ selectedView! !

!SViewWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 14:38'!
selectedView: anObject
	"Set the value of selectedView"

	selectedView := anObject! !

!SViewWrapper methodsFor: 'evaluating' stamp: 'mp 5/5/2010 12:19'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| viewString |
	self tracer isActive
		ifTrue: [
			self 
				beforeMethod;
				beforeMethod: anObject.
			self tracer stopTracing.
			viewString := self getViewString: anObject.
			self tracer addView: viewString to: self selectedView.
			self tracer startTracing.
			[^ self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects] 
				ensure: [self afterMethod]]
		ifFalse: [^ self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects].


! !

MwMethodWrapper subclass: #SMethodByteCodeCoverageForAllTestsWrapper
	instanceVariableNames: 'currentTest coveredByteCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SMethodByteCodeCoverageForAllTestsWrapper class methodsFor: 'instance creation' stamp: 'mp 11/5/2010 11:39:17.439'!
on: aMethodReference
	
	^ self on: aMethodReference methodSymbol inClass: aMethodReference actualClass! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:30.532'!
coveredByteCodes
	"Answer the value of coveredByteCodes"

	^ coveredByteCodes ifNil: [coveredByteCodes := Dictionary new]! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.362'!
coveredByteCodes: anObject
	"Set the value of coveredByteCodes"

	coveredByteCodes := anObject! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.35'!
currentTest
	"Answer the value of currentTest"

	^ currentTest! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'accessing' stamp: 'mp 11/5/2010 14:23:08.354'!
currentTest: anObject
	"Set the value of currentTest"

	currentTest := anObject! !

!SMethodByteCodeCoverageForAllTestsWrapper methodsFor: 'evaluating' stamp: 'mp 11/5/2010 14:40:24.301'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result byteCodes |
	result := thisContext
		runSimulatedMethodOnly: [self clientMethod 
										valueWithReceiver: anObject 
										arguments: anArrayOfObjects]
		contextAtEachStep: [ :ctx | 
			self compiledMethod == ctx method
				ifTrue: [	byteCodes := self coveredByteCodes 
											at: self currentTest ifAbsentPut: [Set new].
						byteCodes add: ctx pc]].
	^ result! !

MwMethodWrapper subclass: #SSpecificWrapper
	instanceVariableNames: 'numberOfCallsBefore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

SSpecificWrapper subclass: #SByteCodeCoverageWrapper
	instanceVariableNames: 'coveredByteCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SByteCodeCoverageWrapper methodsFor: 'accessing' stamp: 'mp 11/8/2010 15:21:28.79'!
coveredByteCodes
	"Answer the value of coveredByteCodes"

	^ coveredByteCodes ifNil: [coveredByteCodes := OrderedCollection new]! !

!SByteCodeCoverageWrapper methodsFor: 'accessing' stamp: 'mp 11/8/2010 15:19:07.504'!
coveredByteCodes: anObject
	"Set the value of coveredByteCodes"

	coveredByteCodes := anObject! !

!SByteCodeCoverageWrapper methodsFor: 'evaluating' stamp: 'mp 11/8/2010 15:54:55.976'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result |
	self isSpecificCall 
		ifTrue: [self decrementCallCount.
				result := thisContext
							runSimulatedMethodOnly: [self clientMethod 
															valueWithReceiver: anObject 
															arguments: anArrayOfObjects]
							contextAtEachStep: [ :ctx | 
								self compiledMethod == ctx method
									ifTrue: [	self coveredByteCodes add: ctx pc]]]
		ifFalse: [self decrementCallCount.
				result := self clientMethod 
								valueWithReceiver: anObject 
								arguments: anArrayOfObjects].
	^ result! !

SSpecificWrapper subclass: #SDebugWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SDebugWrapper methodsFor: 'as yet unclassified' stamp: 'mp 7/7/2010 17:00:04.386'!
beforeMethod

	self numberOfCallsBefore = 1
		ifTrue: [SBreakpointManager installInClass: self wrappedClass selector: self clientSelector.].
	self decrementCallCount.! !

SSpecificWrapper subclass: #SExploreWrapper
	instanceVariableNames: 'objects exploreWhat found'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Tracer'!

!SExploreWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 17:37'!
on: aMethodSelector inClass: aClass waitingCalls: aNumber exploreWhat: aSymbol

	| wrapper |
	wrapper := self on: aMethodSelector inClass: aClass waitingCalls: aNumber.
	wrapper ifNotNil: [wrapper exploreWhat: aSymbol].
	^ wrapper! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
exploreWhat
	"Answer the value of exploreWhat"

	^ exploreWhat! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
exploreWhat: anObject
	"Set the value of exploreWhat"

	exploreWhat := anObject! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:48'!
found
	"Answer the value of found"

	^ found ifNil: [found := false]! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 18:48'!
found: anObject
	"Set the value of found"

	found := anObject! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 1/25/2010 17:34'!
getArguments: anArrayOfObjects

	(self isSpecificCall and: [self exploreWhat == #arguments])
		ifTrue: [self objects add: anArrayOfObjects veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 3/5/2010 16:51'!
getReturn: anObject

	(self isSpecificCall and: [self exploreWhat == #return])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'bs 7/17/2010 14:32:28.652'!
getSelfAfter: anObject
	(self isSpecificCall and: [self exploreWhat == #selfAfter])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 1/25/2010 17:35'!
getSelfBefore: anObject

	(self isSpecificCall and: [self exploreWhat == #selfBefore])
		ifTrue: [self objects add: anObject veryDeepCopy].! !

!SExploreWrapper methodsFor: 'exploring' stamp: 'mp 3/4/2010 09:50'!
getTypeInformation: anArrayOfObjects of: aSymbol

	| type |
	((self isSpecificCall and: [self exploreWhat == #types]) and: [anArrayOfObjects isEmpty not])
		ifTrue: [	anArrayOfObjects size = 1
					ifTrue: [type := anArrayOfObjects first class]
					ifFalse: [type := OrderedCollection new.
							anArrayOfObjects do: [:each | type add: each class]].
				self objects add: aSymbol -> type].! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
objects
	"Answer the value of objects"

	^ objects ifNil: [objects := OrderedCollection new]! !

!SExploreWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 17:30'!
objects: anObject
	"Set the value of objects"

	objects := anObject! !

!SExploreWrapper methodsFor: '-- all --' stamp: 'bs 7/17/2010 16:00:15.791'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	| result |
	"#arguments #selfBefore #selfAfter #return #types"	
	self 
		getTypeInformation: anArrayOfObjects of: #arguments;
		getTypeInformation: (Array with: anObject) of: #self;
		getArguments: anArrayOfObjects;
		getSelfBefore: anObject.
	self isSpecificCall ifTrue: [self found: true].
	self decrementCallCount.
	^ [
		result := self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
		result.
	] ensure: [ self found 
				ifTrue: [self incrementCallCount.
					self 
						getSelfAfter: anObject;
						getReturn: result;
						getTypeInformation: (Array with: result) of: #result.
					self isSpecificCall 
						ifTrue: [self found: false. 
							      self decrementCallCount]]]
! !

!SSpecificWrapper class methodsFor: 'testing' stamp: 'mp 4/1/2011 15:57:42.651'!
canWrap: aSymbol inClass: aClass
	(aClass inheritsFrom: ACUserStory) ifTrue: [^ false].
	^ super canWrap: aSymbol inClass: aClass! !

!SSpecificWrapper class methodsFor: 'instance creation' stamp: 'mp 1/25/2010 16:40'!
on: aSymbol inClass: aClass waitingCalls: aNumber

	| wrapper |
	wrapper := self on: aSymbol inClass: aClass.
	wrapper ifNotNil: [wrapper numberOfCallsBefore: aNumber].
	^ wrapper! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 16:39'!
decrementCallCount
	
	self numberOfCallsBefore: self numberOfCallsBefore - 1.! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 19:07'!
incrementCallCount
	
	self numberOfCallsBefore: self numberOfCallsBefore + 1.! !

!SSpecificWrapper methodsFor: 'count' stamp: 'mp 1/25/2010 16:40'!
isSpecificCall
	
	^ self numberOfCallsBefore = 0! !

!SSpecificWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 16:39'!
numberOfCallsBefore
	"Answer the value of numberOfCallsBefore"

	^ numberOfCallsBefore! !

!SSpecificWrapper methodsFor: 'accessing' stamp: 'mp 1/25/2010 16:39'!
numberOfCallsBefore: anObject
	"Set the value of numberOfCallsBefore"

	numberOfCallsBefore := anObject! !

SCodeExplorerIndentingListItemMorph subclass: #SDebuggerCodeExplorerIndentingListItemMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Finder-Debugger'!

!SDebuggerCodeExplorerIndentingListItemMorph methodsFor: 'accessing' stamp: 'mp 11/8/2010 16:32:22.369'!
codePane
	"Answer the value of codePane"
	| suspiciousColor |
	^ codePane ifNil: [
		self icon ifNotNil: [suspiciousColor := self icon colorAt: 1@1].
		codePane := SDebuggerCodePaneMorph 
										on: self callNode 
										color: self getBackgroundColor
										suspiciousColor: suspiciousColor].! !
